@@ -738,12 +738,19 @@ env2d_basic_plot <- function(x, var = c('obs', 'lo', 'hi', 'lo.sign', 'hi.sign')
 globalVariables(c("main", "label"))
 
 # A helper function for env2d_ggplot2
-# Pixel size: w = rx[2]-rx[1], h = ry[2]-ry[1]
-#' @importFrom ggplot2 ggplot geom_tile aes geom_contour coord_fixed .data labs
-env2d_ggplot2_helper_1 <- function(df, w, h, sign.col, transparency, contours = TRUE) {
-  g <- ggplot() + geom_tile(data=df, aes(x=.data$x, y=.data$y, fill=.data$z), width=w, height=h)
-  g <- g + geom_tile(data=df[df$signif,], aes(x=.data$x, y=.data$y), fill=sign.col, alpha=transparency, width=w, height=h)
-  if(contours) g <- g + geom_contour(data=df[df$contour,], aes(x=.data$x, y=.data$y, z=.data$z))
+#' @importFrom ggplot2 ggplot geom_tile geom_rect aes geom_contour coord_fixed .data labs
+env2d_ggplot2_helper_1 <- function(df, sign.col, transparency, contours = TRUE) {
+  if(!is.null(df$xmin)) {
+    aesxy <- aes(xmin=.data$xmin, ymin=.data$ymin, xmax=.data$xmax, ymax=.data$ymax)
+    geom <- geom_rect
+  } else {
+    aesxy <- aes(x=.data$x, y=.data$y, width=.data$width, height=.data$height)
+    geom <- geom_tile
+  }
+  g <- ggplot(df, aesxy)
+  g <- g + geom(aes(fill=.data$z))
+  g <- g + geom(data=df[df$signif,], aesxy, fill=sign.col, alpha=transparency)
+  if(contours && !is.null(df$x)) g <- g + geom_contour(data=df[df$contour,], aes(x=.data$x, y=.data$y, z=.data$z))
   g <- g + coord_fixed(ratio=1)
   g <- g + labs(x="", y="", fill="")
   g
@@ -763,7 +770,22 @@ env2d_ggplot2_helper <- function(x, fixedscales, contours = TRUE, main="", inser
     }
   }
 
-  df <- expand.grid(x=x$r[[1]], y=x$r[[2]])
+  if(all(sapply(x$r, length) == length(x$obs))) {
+    df <- as.data.frame(x$r)
+  } else {
+    if(!is.null(x$r$xmin)) {
+      df <- expand.grid(xmin=x$r$xmin, ymin=x$r$ymin)
+      df <- cbind(df, expand.grid(xmax=x$r$xmax, ymax=x$r$ymax))
+    } else {
+      df <- expand.grid(x=x$r$x, y=x$r$y)
+      if(!is.null(x$r$width)) {
+        df <- cbind(df, expand.grid(width=x$r$width, height=x$r$height))
+      } else {
+        df$width <- x$r$x[2] - x$r$x[1]
+        df$height <- x$r$y[2] - x$r$y[1]
+      }
+    }
+  }
   adddf <- function(df, z, name, label=namelist[[name]], contour=FALSE, signif=FALSE) {
     df$z <- c(z)
     df$name <- name
@@ -799,7 +821,7 @@ env2d_ggplot2_helper <- function(x, fixedscales, contours = TRUE, main="", inser
 # @param transparency A number between 0 and 1.
 # Similar to alpha of \code{\link[grDevices]{rgb}}. Used in plotting the significant regions.
 #' @importFrom ggplot2 theme element_blank facet_wrap vars
-env2d_ggplot2_helper_many_single_plots <- function(dfs, w, h, sign.col, transparency, contours = TRUE) {
+env2d_ggplot2_helper_many_single_plots <- function(dfs, sign.col, transparency, contours = TRUE) {
   remove_axes_theme <- theme(axis.title.x=element_blank(),
                              axis.text.x=element_blank(),
                              axis.ticks.x=element_blank(),
@@ -807,7 +829,7 @@ env2d_ggplot2_helper_many_single_plots <- function(dfs, w, h, sign.col, transpar
                              axis.text.y=element_blank(),
                              axis.ticks.y=element_blank())
   lapply(dfs, function(df) {
-    g <- env2d_ggplot2_helper_1(df, w, h, sign.col, transparency, contours)
+    g <- env2d_ggplot2_helper_1(df, sign.col, transparency, contours)
     g <- g + facet_wrap(vars(label))
     g + remove_axes_theme
   })
